# ------------------------------------------------------------------------
# The following Python code is implemented by Professor Terje Haukaas at
# the University of British Columbia in Vancouver, Canada. It is made
# freely available online at terje.civil.ubc.ca together with notes,
# examples, and additional Python code. Please be cautious when using
# this code; it may contain bugs and comes without warranty of any kind.
# ------------------------------------------------------------------------
# Notation:
# F(x) = objective function, or merit function, relevant in optimization, not in root-finding
# f(x) = dF/dx = derivative of the objective function, i.e., the function whose root is sought
# h(x) = df/dx = d^2F/dx^2 = second-order derivative, i.e., Hessian of the objective function

def armijoLineSearch(F, initialStepSize, maxIterations):

    # Store the incoming step size
    stepSize = initialStepSize

    # Objective function value at initial step size
    Fprevious = F(stepSize)

    # Start the loop
    counter = 0
    convergence = False
    while not convergence:

        # Increment counter
        counter += 1

        # Chop the step size in half
        stepSize = 0.5 * stepSize

        # Objective function one step back
        Fback = F(stepSize)

        # Check if we need to continue chopping
        if Fback > Fprevious or counter == maxIterations:
            optimum = 2*stepSize
            convergence = True

        else:
            Fprevious = Fback

    # Output
    print('\n'"Armijo search done after", counter, "steps with solution", optimum)

    return optimum