# ------------------------------------------------------------------------
# The following Python code is implemented by Professor Terje Haukaas at
# the University of British Columbia in Vancouver, Canada. It is made
# freely available online at terje.civil.ubc.ca together with notes,
# examples, and additional Python code. Please be cautious when using
# this code; it may contain bugs and comes without warranty of any kind.
# ------------------------------------------------------------------------
# Notation:
# F(x) = objective function, or merit function, relevant in optimization, not in root-finding
# f(x) = dF/dx = function whose root is sought
# h(x) = df/dx = d^2F/dx^2 = Hessian of objective function
# ------------------------------------------------------------------------

def f(x, F):

    # Original function value
    objective = F(x)

    # Set the perturbation as a fraction of the x-value
    dx = 0.001
    if x != 0.0:
        dx = dx * x

    # Perturbed function value
    dF = F(x + dx)

    # Sought derivative
    return (dF - objective) / dx


def h(x, F):

    # Original function value
    objective = F(x)

    # Set the perturbation as a fraction of the x-value
    dx = 0.001
    if x != 0.0:
        dx = dx * x

    # Function value, perturbed forward
    dFforward = F(x + dx)

    # Function value, perturbed backward
    dFbackward = F(x - dx)

    # Sought derivative
    return (dFforward - 2.0 * objective + dFbackward) / (dx**2)



