import matplotlib.pyplot as plt
from GroundMotionReadPEERrecord import *
from G2AnalysisSDOFNonlinearDynamic import *
from GroundMotionRecordIntegration import *
from SDOFnewmark import *



directory = 'GroundMotions/ATC63GroundMotions/FarField'    # FarField  /  NearFieldWithPulse  /  NearFieldWithoutPulse
m = 100
counter = 0
totalCounter = 0
for filename in os.listdir(directory):

    # Open and check the file
    f = os.path.join(directory, filename)
    if os.path.isfile(f) and not filename.startswith('.'):

        # Read the record, including delta-t in the header, and append a few zeros at the end
        dt, gm, name = readPEERrecord(f)
        gm = np.pad(gm, (0, int(0.1*len(gm))), 'constant')
        duration = (len(gm)-1) * dt
        gm = np.multiply(gm, 9.81)
        gmVel = trapezoidalRecordIntegration(dt, gm)
        gmDisp = trapezoidalRecordIntegration(dt, gmVel)

        # Plot the ground motion
        plt.ion()
        fig = plt.figure()
        plt.axis('off')
        plt.title("Ground Motion")
        fig.add_subplot(311)
        plt.plot(np.linspace(0, dt * (len(gm)-1), len(gm)), gm, 'k-', linewidth=1.0)
        plt.ylabel("$\ddot{u}_g$")
        fig.add_subplot(312)
        plt.plot(np.linspace(0, dt * (len(gm)-1), len(gm)), gmVel, 'k-', linewidth=1.0)
        plt.ylabel("$\dot{u}_g$")
        fig.add_subplot(313)
        plt.plot(np.linspace(0, dt * (len(gm)-1), len(gm)), gmDisp, 'k-', linewidth=1.0)
        plt.ylabel("$u_g$")
        plt.xlabel("Time [sec.]")

        # Response spectrum analysis
        dampingRatio = 0.03
        minPeriod = 0.05
        maxPeriod = 10
        numSpectrumPoints = 200
        displacementSpectrum = []
        velocitySpectrum = []
        accelerationSpectrum = []
        periodAxis = []

        for i in range(numSpectrumPoints):

            period = minPeriod + i * maxPeriod / numSpectrumPoints

            periodAxis.append(period)

            k = 4 * m * np.pi ** 2 / period ** 2

            c = 2 * np.sqrt(k * m) * dampingRatio

            [disp, vel, accel] = sdofNewmark(m, c, k, dt, gm)

            displacementSpectrum.append(np.max(np.abs(disp)))
            velocitySpectrum.append(np.max(np.abs(vel)))
            accelerationSpectrum.append(np.max(np.abs(accel))/9.81)

        plt.ion()
        fig = plt.figure()
        plt.axis('off')
        plt.title('Response spectra')

        axes = fig.add_subplot(311)
        plt.loglog(periodAxis, accelerationSpectrum, 'k-', linewidth=1.0)
        plt.ylabel('$S_a [g]$')

        axes = fig.add_subplot(312)
        plt.plot(periodAxis, velocitySpectrum, 'k-', linewidth=1.0)
        plt.ylabel('$S_v$')

        axes = fig.add_subplot(313)
        plt.plot(periodAxis, displacementSpectrum, 'k-', linewidth=1.0)
        plt.ylabel('$S_d$')
        plt.xlabel('Period [sec]')

        print('\n'"Click somewhere in the plot to continue...")
        plt.waitforbuttonpress()
