# ------------------------------------------------------------------------
# The following Python code is implemented by Professor Terje Haukaas at
# the University of British Columbia in Vancouver, Canada. It is made
# freely available online at terje.civil.ubc.ca together with notes,
# examples, and additional Python code. Please be cautious when using
# this code; it may contain bugs and comes without warranty of any kind.
# ------------------------------------------------------------------------

import matplotlib.pyplot as plt

def trapezoidalModulatingFunction(t, t1, t2, t3, t4):
    if t < t1:
        return 0.0
    elif t < t2:
        return (t - t1) / (t2 - t1)
    elif t < t3:
        return 1.0
    elif t < t4:
        return 1.0 - (t - t3) / (t4 - t3)
    else:
        return 0.0

def modulate(dt, gm, t1, t2, t3, t4, type="Trapezoidal"):

    t = []
    modFcn = []
    gmModulated = []

    for i in range(len(gm)):

        time = i*dt
        modFcnValue = trapezoidalModulatingFunction(time, t1, t2, t3, t4)

        t.append(time)
        modFcn.append(modFcnValue)
        gmModulated.append(gm[i]*modFcnValue)

        plt.ion()
        plt.figure()
        plt.plot(t, modFcn, 'k-')
        plt.xlabel("Time")
        plt.title("Modulating Function")

        plt.ion()
        plt.figure()
        plt.plot(t, gmModulated, 'k-')
        plt.xlabel("Time")
        plt.title("Modulated Ground Acceleration")
