# ------------------------------------------------------------------------
# The following Python code is implemented by Professor Terje Haukaas at
# the University of British Columbia in Vancouver, Canada. It is made
# freely available online at terje.civil.ubc.ca together with notes,
# examples, and additional Python code. Please be cautious when using
# this code; it may contain bugs and comes without warranty of any kind.
# ------------------------------------------------------------------------

def sdofNewmark(m, c, k, delta_t, record):

    beta = 1.0/6.0
    F1 = 1.0/beta*(m/delta_t+c/2.0)
    F2 = 1.0/2.0*(c*(2.0-1.0/(2.0*beta))*delta_t-m/beta)
    F3 = 1.0/beta*(m/delta_t**2+c/(2.0*delta_t))
    F4 = 1.0/(beta*delta_t**2)
    F5 = 1.0/(beta*delta_t)
    F6 = 1.0/(2.0*beta)
    F7 = 1.0/(2.0*beta*delta_t)
    F8 = 1.0/2.0*(2.0-1.0/(2.0*beta))*delta_t

    structuralDisplacement = [0]
    structuralVelocity = [0]
    structuralAcceleration = [0]

    time = 0.0
    ui = 0.0
    uDoti = 0.0
    uDotDoti = 0.0
    for i in range(len(record)-1):

        delta_force = -m * (record[i+1]-record[i])

        delta_displacement = (delta_force + F1*uDoti - F2*uDotDoti) / (F3 + k)

        delta_velocity = F7*delta_displacement - F6*uDoti + F8*uDotDoti

        delta_acceleration = F4*delta_displacement - F5*uDoti - F6*uDotDoti

        ui += delta_displacement

        uDoti += delta_velocity

        uDotDoti += delta_acceleration

        time += delta_t

        # Store *relative* disp & vel but *total* acceleration
        structuralDisplacement.append(ui)
        structuralVelocity.append(uDoti)
        structuralAcceleration.append(uDotDoti + record[i+1])

    return [structuralDisplacement, structuralVelocity, structuralAcceleration]